<?php

namespace Drupal\lb_tabs\Plugin\Layout;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\LayoutDefault;

abstract class LbTabsLayoutBase extends LayoutDefault {

  /**
   * Maximum item count so that a huge input does not hang up the system.
   */
  const MAX_ITEMS = 99;

  /**
   * The ID prefix.
   */
  const HTML_ID_PREFIX = 'OVERRIDE_ME';

  /**
   * {@inheritDoc}
   */
  public function build(array $regions) {
    $this->setPluginDefinitionRegions();
    $build = parent::build($regions);
    if ($this->configuration['labels_from_title']) {
      $this->setLabelsFromBlockTitles($build, $regions);
    }
    // @aee template_preprocess_layout
    $build['#settings']['dom_id'] = Html::getUniqueId(static::HTML_ID_PREFIX);
    return $build;
  }

  /**
   * Set our dynamic regions.
   */
  protected function setPluginDefinitionRegions() {
    $regionMap = [];
    foreach (range(1, $this->configuration['item_count']) as $i) {
      if (!$this->configuration['labels_from_title']) {
        $regionMap["label_$i"] = ['label' => $this->t('Label @i', ['@i' => $i])];
      }
      $regionMap["content_$i"] = ['label' => $this->t('Content @i', ['@i' => $i])];
    }
    $this->pluginDefinition->setRegions($regionMap);
  }

  /**
   * Set labels from block titles.
   *
   * @param array $build
   *   The build.
   * @param array $regions
   *   The regions array.
   */
  protected function setLabelsFromBlockTitles(array &$build, array $regions) {
    foreach (range(1, $this->configuration['item_count']) as $i) {
      $region = $regions["content_$i"];
      unset($label);
      foreach ($region as $block) {
        if (!empty($block['#configuration']['label'])) {
          $label = $block['#configuration']['label'];
          break;
        }
      }
      $build["label_$i"] = ['#plain_text' => $label ?? $i];
    }
}

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
        'item_count' => 1,
        'initially_active_item' => NULL,
        'labels_from_title' => FALSE,
      ];
  }

  /**
   * {@inheritDoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['item_count'] = [
      '#type' => 'number',
      '#min' => 1,
      '#max' => static::MAX_ITEMS,
      '#step' => 1,
      '#required' => TRUE,
      '#title' => $this->t('No. of items'),
      '#default_value' => $this->configuration['item_count'],
    ];
    $form['initially_active_item'] = [
      '#type' => 'number',
      '#min' => 1,
      '#max' => static::MAX_ITEMS,
      '#step' => 1,
      '#title' => $this->t('Initially active item'),
      '#description' => $this->t('The item that is active initially. Leave empty for none.'),
      '#default_value' => $this->configuration['initially_active_item'],
    ];
    $form['second_click_closes'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Second click closes'),
      '#description' => $this->t('If set, a second click closes an item.'),
      '#default_value' => $this->configuration['second_click_closes'],
    ];
    $form['labels_from_title'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use block titles as labels'),
      '#description' => $this->t('Use title of first block for label. Otherwise we have separate regions for label, giving far more power.'),
      '#default_value' => $this->configuration['labels_from_title'],
    ];
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['item_count'] = $form_state->getValue('item_count');
    $this->configuration['initially_active_item'] = $form_state->getValue('initially_active_item');
    $this->configuration['second_click_closes'] = $form_state->getValue('second_click_closes');
    $this->configuration['labels_from_title'] = $form_state->getValue('labels_from_title');
  }

}
